package sun.nio.cs;

import java.nio.charset.Charset;
import java.nio.charset.CharsetEncoder;
import java.nio.charset.CharsetDecoder;

public class ThreadLocalCoders
{
    private static final int CACHE_SIZE = 3;
    private static ThreadLocalCoders.Cache decoderCache;
    private static ThreadLocalCoders.Cache encoderCache;
    
    public ThreadLocalCoders() {
    }
    
    public static CharsetDecoder decoderFor(Object o) {
        CharsetDecoder charsetDecoder = (CharsetDecoder)ThreadLocalCoders.decoderCache.forName(o);
        charsetDecoder.reset();
        return charsetDecoder;
    }
    
    public static CharsetEncoder encoderFor(Object o) {
        CharsetEncoder charsetEncoder = (CharsetEncoder)ThreadLocalCoders.encoderCache.forName(o);
        charsetEncoder.reset();
        return charsetEncoder;
    }
    
    static {
        ThreadLocalCoders.decoderCache = new ThreadLocalCoders.Cache(3) {
            @Override
            boolean hasName(Object o, Object o2) {
                if ((o2 instanceof String)) {
                    return ((CharsetDecoder)o).charset().name().equals(o2);
                }
                return o2 instanceof Charset && ((CharsetDecoder)o).charset().equals(o2);
            }
            
            @Override
            Object create(Object o) {
                if ((o instanceof String)) {
                    return Charset.forName((String)o).newDecoder();
                }
                if ((o instanceof Charset)) {
                    return ((Charset)o).newDecoder();
                }
                assert false;
                return null;
            }
        };
        ThreadLocalCoders.encoderCache = new ThreadLocalCoders.Cache(3) {
            @Override
            boolean hasName(Object o, Object o2) {
                if ((o2 instanceof String)) {
                    return ((CharsetEncoder)o).charset().name().equals(o2);
                }
                return o2 instanceof Charset && ((CharsetEncoder)o).charset().equals(o2);
            }
            
            @Override
            Object create(Object o) {
                if ((o instanceof String)) {
                    return Charset.forName((String)o).newEncoder();
                }
                if ((o instanceof Charset)) {
                    return ((Charset)o).newEncoder();
                }
                assert false;
                return null;
            }
        };
    }
    
    private abstract static class Cache
    {
        private ThreadLocal<Object[]> cache;
        private final int size;
        
        Cache(int size) {
            this.cache = new ThreadLocal();
            this.size = size;
        }
        
        abstract Object create(Object p0);
        
        private void moveToFront(Object[] array, int i) {
            Object o = array[i];
            for (; i > 0; --i) {
                array[i] = array[i - 1];
            }
            array[0] = o;
        }
        
        abstract boolean hasName(Object p0, Object p1);
        
        Object forName(Object o) {
            Object[] value = (Object[])this.cache.get();
            if (value == null) {
                value = new Object[this.size];
                this.cache.set(value);
            }
            else {
                for (int i = 0; i < value.length; ++i) {
                    Object o2 = value[i];
                    if (o2 != null && this.hasName(o2, o)) {
                        if (i > 0) {
                            this.moveToFront(value, i);
                            return o2;
                        }
                        return o2;
                    }
                }
            }
            Object create = this.create(o);
            value[value.length - 1] = create;
            this.moveToFront(value, value.length - 1);
            return create;
        }
    }
}
